@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2018-2021 Vaclav Slavik
+ *  Copyright (C) 2018-2022 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -580,7 +580,9 @@ public:
         m_string = str::to_wx(extractor.extractedText);
 
         // TODO: switch to textual IDs in CatalogItem
-        std::string id = node.attribute("id").value();
+        std::string id = node.attribute("resname").value();
+        if (id.empty())
+            id = node.attribute("id").value();
         // some tools (e.g. Xcode, tool-id="com.apple.dt.xcode") use ID same as text
         if (!id.empty() && id != m_string)
             m_extractedComments.push_back("ID: " + str::to_wx(id));
@@ -757,7 +759,9 @@ public:
         m_string = str::to_wx(extractor.extractedText);
 
         // TODO: switch to textual IDs in CatalogItem
-        std::string id = unit().attribute("id").value();
+        std::string id = unit().attribute("name").value();
+        if (id.empty())
+            id = unit().attribute("id").value();
         // some tools (e.g. Xcode, tool-id="com.apple.dt.xcode") use ID same as text
         if (!id.empty() && id != m_string)
             m_extractedComments.push_back("ID: " + str::to_wx(id));
