@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2021 Vaclav Slavik
+ *  Copyright (C) 1999-2022 Vaclav Slavik
  *  Copyright (C) 2005 Olivier Sannier
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
@@ -78,6 +78,11 @@ class PoeditListCtrl : public wxDataViewCtrl
             return m_model->GetItem(m_model->RowFromCatalogIndex(index));
         }
 
+        wxDataViewItem CatalogItemToListItem(const CatalogItemPtr& item) const
+        {
+            return m_model->GetItem(m_model->RowFromCatalogItem(item));
+        }
+
         /// Returns item's index in the catalog
         int ListIndexToCatalog(int index) const
         {
@@ -278,6 +283,21 @@ class PoeditListCtrl : public wxDataViewCtrl
                     return m_mapCatalogToList[index];
             }
 
+            int RowFromCatalogItem(const CatalogItemPtr& item) const
+            {
+                if (!item)
+                    return -1;
+
+                auto& items = m_catalog->items();
+                for (size_t i = 0; i < items.size(); i++)
+                {
+                    if (items[i] == item)
+                        return RowFromCatalogIndex((int)i);
+                }
+
+                return -1;
+            }
+
             void CreateSortMap();
 
             void Freeze() { m_frozen = true; }
@@ -306,6 +326,7 @@ class PoeditListCtrl : public wxDataViewCtrl
         void UpdateHeaderAttrs();
         void CreateColumns();
         void UpdateColumns();
+        void FixIdColumnSize();
         void OnSize(wxSizeEvent& event);
 
         bool m_displayIDs;
