@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2021 Vaclav Slavik
+ *  Copyright (C) 1999-2022 Vaclav Slavik
  *  Copyright (C) 2015 PrismJS (CSS parts)
  *  Copyright (c) 2013-2017 Cole Bemis (Feather Icons)
  *
@@ -136,9 +136,23 @@ FileViewer::FileViewer(wxWindow*)
 #endif
     m_topBarSizer->Add(m_openInEditor, wxSizerFlags().Center().ReserveSpaceEvenIfHidden().Border(wxLEFT, PX(10)));
 
-    sizer->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxSize(-1, 1)), wxSizerFlags().Expand().Border(wxLEFT|wxRIGHT, PX(5)));
+    sizer->Add(new wxStaticLine(panel, wxID_ANY, wxDefaultPosition, wxSize(-1, 1)), wxSizerFlags().Expand().Border(wxLEFT|wxRIGHT, PX(5)));
 
+#ifdef __WXMSW__
+    if (wxWebView::IsBackendAvailable(wxWebViewBackendEdge))
+    {
+        m_usesMSIE = false;
+        m_content = wxWebView::New(panel, wxID_ANY);
+    }
+    else
+    {
+        m_usesMSIE = true;
+        wxWebViewIE::MSWSetEmulationLevel(wxWEBVIEWIE_EMU_IE11);
+        m_content = wxWebView::New(panel, wxID_ANY);
+    }
+#else
     m_content = wxWebView::New(panel, wxID_ANY);
+#endif
     sizer->Add(m_content, 1, wxEXPAND);
 
     RestoreWindowState(this, wxSize(PX(600), PX(400)));
@@ -303,8 +317,7 @@ void FileViewer::ShowHTMLContent(const wxString& markup)
     Layout();
 
 #ifdef __WXMSW__
-    // On Windows, we use embedded MSIE browser (Edge embedding is still a bit
-    // experimental). Streaming document content to it via SetPage() behaves
+    // Streaming document content to MSIE it via SetPage() behaves
     // a bit differently from loading a file or HTTP document and in particular,
     // we're hit by two issues:
     //
@@ -315,6 +328,7 @@ void FileViewer::ShowHTMLContent(const wxString& markup)
     //
     // So we instead put the content into a temporary file and load that. This
     // sidesteps both issues at a negligible performance cost.
+    if (m_usesMSIE)
     {
         if (!m_tmpFile)
             m_tmpFile = std::make_shared<TempFile>();
@@ -325,9 +339,11 @@ void FileViewer::ShowHTMLContent(const wxString& markup)
 
         m_content->LoadURL(wxFileName::FileNameToURL(m_tmpFile->file));
     }
-#else
-    m_content->SetPage(markup, "file:///");
+    else
 #endif
+    {
+        m_content->SetPage(markup, "file:///");
+    }
 }
 
 
@@ -343,9 +359,10 @@ void FileViewer::ShowError(const char *icon, const wxString& msg, const wxString
                 <style>%s</style>
             </head>
             <body>
-                <div class="message">
-                    <h1>%s</h1>
-                    <h2>%s</h2>
+                <div class="message-wrapper">
+                    <div class="message">
+                        <h1>%s</h1>
+                        <h2>%s</h2>
         )",
         HTML_POEDIT_CSS,
         icon,
@@ -359,6 +376,7 @@ void FileViewer::ShowError(const char *icon, const wxString& msg, const wxString
         html += wxString::Format("<div class=\"explanation\">%s</div>", EscapeMarkup(description));
 
     html += R"(
+                    </div>
                 </div>
             </body>
         </html>
@@ -445,7 +463,7 @@ wxString FileToHTMLMarkup(const wxTextFile& file, const wxString& ext, size_t li
     for (size_t i = 0; i < count; i++)
     {
         if (i == lineno-1)
-            html += "<span id=\"mark\"></span>";
+            html += "<span id=\"mark\"><span id=\"msie_anchor\"></span></span>";
         else
             html += "<span></span>";
     }
@@ -453,10 +471,15 @@ wxString FileToHTMLMarkup(const wxTextFile& file, const wxString& ext, size_t li
     html += "</span></code></pre>";
 
     if (lineno)
+    {
+        // Alternative implementation that doesn't need msie_anchor, but doesn't work on MSIE, is to do:
+        //     document.getElementById('mark').scrollIntoView({behavior: 'instant', block: 'center'});
+        // instead of
+        //     window.location.hash = "#msie_anchor";
         html += R"(<script>
-                document.getElementById('mark').scrollIntoView({behavior: 'instant', block: 'center'});
-                if (document.documentMode) window.scrollBy(0, -100); // MSIE
-                </script>)";
+                   window.location.hash = "#msie_anchor";
+                   </script>)";
+    }
 
     // PrismJS is added after everything else so that basic rendering works even
     // when offline.
@@ -744,8 +767,22 @@ mark {
     }
 }
 
+#msie_anchor {
+    display: block;
+    visibility: hidden;
+    height: 50vh; /* 50% viewport height */
+    margin-top: -50vh;
+}
+
 /* Error messages: */
 
+.message-wrapper {
+    height: 90vh;
+    display: flex;
+    align-items: center;
+    justify-content: center;
+}
+
 .message {
     text-align: center;
     opacity: 0.8;
