@@ -520,7 +520,7 @@ class AnyTranslatableTextCtrl::Attributes
 {
 public:
 #ifdef __WXOSX__
-    NSDictionary *m_attrSpace, *m_attrEscape, *m_attrMarkup, *m_attrFormat;
+    NSDictionary *m_attrSpace, *m_attrEscape, *m_attrMarkup, *m_attrPlaceholder;
     typedef NSDictionary* AttrType;
 
     Attributes(wxTextCtrl*)
@@ -529,10 +529,10 @@ public:
         m_attrEscape = @{NSBackgroundColorAttributeName: ColorScheme::Get(Color::SyntaxEscapeBg).OSXGetNSColor(),
                          NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxEscapeFg).OSXGetNSColor()};
         m_attrMarkup = @{NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxMarkup).OSXGetNSColor()};
-        m_attrFormat = @{NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxFormat).OSXGetNSColor()};
+        m_attrPlaceholder = @{NSForegroundColorAttributeName: ColorScheme::Get(Color::SyntaxFormat).OSXGetNSColor()};
     }
 #else // !__WXOSX__
-    wxTextAttr m_attrDefault, m_attrSpace, m_attrEscape, m_attrMarkup, m_attrFormat;
+    wxTextAttr m_attrDefault, m_attrSpace, m_attrEscape, m_attrMarkup, m_attrPlaceholder;
     typedef wxTextAttr AttrType;
 
     Attributes(wxTextCtrl *ctrl)
@@ -547,7 +547,7 @@ public:
 
         m_attrMarkup.SetTextColour(ColorScheme::Get(Color::SyntaxMarkup));
 
-        m_attrFormat.SetTextColour(ColorScheme::Get(Color::SyntaxFormat));
+        m_attrPlaceholder.SetTextColour(ColorScheme::Get(Color::SyntaxFormat));
     }
 
     const AttrType& Default() const {  return m_attrDefault; }
@@ -560,7 +560,7 @@ public:
             case SyntaxHighlighter::LeadingWhitespace:  return m_attrSpace;
             case SyntaxHighlighter::Escape:             return m_attrEscape;
             case SyntaxHighlighter::Markup:             return m_attrMarkup;
-            case SyntaxHighlighter::Format:             return m_attrFormat;
+            case SyntaxHighlighter::Placeholder:        return m_attrPlaceholder;
         }
         return m_attrSpace; // silence bogus warning
     }
@@ -824,12 +824,28 @@ void AnyTranslatableTextCtrl::UpdateRTLStyle()
 void AnyTranslatableTextCtrl::HighlightText()
 {
 #ifdef __WXOSX__
-
     // See the comment in DoGetValueForRange() for why GetValue() returns subtly
     // different thing in RTL.
     // For highlighting, where we index into the string, we need to operate on the
     // exact same string addTemporaryAttributes:forCharacterRange: is expecting.
-    auto text = str::to_wstring([TextView(this) string]);
+    auto traw = [TextView(this) string];
+    auto text = str::to_wstring(traw);
+    if (text.length() != [traw length] )
+    {
+        // Internally, NSString uses UTF-16 and all indexes including via NSLayoutManager
+        // are in it. std::wstring is 32bit and uses UCS4/UTF-32, so simply using wstring
+        // with syntax highlighter would break on any string with characters outside of BMP
+        // (e.g. Emoji, see https://github.com/vslavik/poedit/issues/731).
+        //
+        // To fix this, we need to guarantee same indices in NSString/UTF-16 and wstring,
+        // and a simple way to do it is to put UTF-16 data into wstring. That of course yields
+        // incorrectly encoded string, but that doesn't matter ofr the purpose of syntax
+        // highlighting, which is safely within BMP.
+        //
+        // Only do this in the rare non-BMP case for efficiency.
+        std::u16string utf16 = boost::locale::conv::utf_to_utf<char16_t>([traw UTF8String]);
+        text = std::wstring(utf16.begin(), utf16.end());
+    }
 
     NSRange fullRange = NSMakeRange(0, text.length());
     NSLayoutManager *layout = [TextView(this) layoutManager];
